--
-- FlightMap - AddOn to show inbound and outbound flightpaths from a given
--             zone on the World Map.  Additionally shows flight costs and
--             zone level ranges.
-- Copyright (c) 2005 Byron Ellacott (Dhask of Uther)
--
-- An unlimited license to use, reproduce and copy this work is granted, on
-- the condition that the licensee accepts all responsibility and liability
-- for any damage that may arise from the use of this AddOn.

-- Version number, updated by Subversion
FLIGHTMAP_VERSION   = string.gsub("$Rev: 99 $", '%$R.v: (%d+) %$', '%1');

-- Maximum lines to draw at once
FLIGHTMAP_MAX_PATHS = 15;

-- Size and names for path texture files
FLIGHTMAP_LINE_SIZE = 256;
FLIGHTMAP_TEX_UP    = "Interface\\AddOns\\FlightMap\\FlightMapUp";
FLIGHTMAP_TEX_DOWN  = "Interface\\AddOns\\FlightMap\\FlightMapDown";

-- Maximum POI buttons defined
FLIGHTMAP_MAX_POIS  = 15;

-- How many pixels is too close to another POI?
FLIGHTMAP_CLOSE     = 16;

-- Textures for flightmaster POI icons
FLIGHTMAP_POI_KNOWN = "Interface\\TaxiFrame\\UI-Taxi-Icon-Green";
FLIGHTMAP_POI_OTHER = "Interface\\TaxiFrame\\UI-Taxi-Icon-Gray";

local lTYPE_HORDE     = FLIGHTMAP_HORDE;
local lTYPE_ALLIANCE  = FLIGHTMAP_ALLIANCE;
local lTYPE_CONTESTED = FLIGHTMAP_CONTESTED;

-- According to http://www.worldofwarcraft.com/ at any rate...
FLIGHTMAP_RANGES   = {
    [FLIGHTMAP_ELWYNN]        = { 1, 10, lTYPE_ALLIANCE},
    [FLIGHTMAP_DUNMOROGH]     = { 1, 10, lTYPE_ALLIANCE},
    [FLIGHTMAP_TIRISFAL]      = { 1, 10, lTYPE_HORDE},
    [FLIGHTMAP_LOCHMODAN]     = {10, 20, lTYPE_ALLIANCE},
    [FLIGHTMAP_SILVERPINE]    = {10, 20, lTYPE_HORDE},
    [FLIGHTMAP_WESTFALL]      = {10, 20, lTYPE_ALLIANCE},
    [FLIGHTMAP_REDRIDGE]      = {15, 25, lTYPE_CONTESTED},
    [FLIGHTMAP_DUSKWOOD]      = {18, 30, lTYPE_CONTESTED},
    [FLIGHTMAP_HILLSBRAD]     = {20, 30, lTYPE_CONTESTED},
    [FLIGHTMAP_WETLANDS]      = {20, 30, lTYPE_CONTESTED},
    [FLIGHTMAP_ALTERAC]       = {30, 40, lTYPE_CONTESTED},
    [FLIGHTMAP_ARATHI]        = {30, 40, lTYPE_CONTESTED},
    [FLIGHTMAP_STRANGLETHORN] = {30, 45, lTYPE_CONTESTED},
    [FLIGHTMAP_BADLANDS]      = {35, 45, lTYPE_CONTESTED},
    [FLIGHTMAP_SORROWS]       = {35, 45, lTYPE_CONTESTED},
    [FLIGHTMAP_HINTERLANDS]   = {40, 50, lTYPE_CONTESTED},
    [FLIGHTMAP_SEARINGGORGE]  = {43, 50, lTYPE_CONTESTED},
    [FLIGHTMAP_BLASTEDLANDS]  = {45, 55, lTYPE_CONTESTED},
    [FLIGHTMAP_BURNINGSTEPPE] = {50, 58, lTYPE_CONTESTED},
    [FLIGHTMAP_WESTERNPLAGUE] = {51, 58, lTYPE_CONTESTED},
    [FLIGHTMAP_EASTERNPLAGUE] = {53, 60, lTYPE_CONTESTED},
    [FLIGHTMAP_DUROTAR]       = { 1, 10, lTYPE_HORDE},
    [FLIGHTMAP_MULGORE]       = { 1, 10, lTYPE_HORDE},
    [FLIGHTMAP_DARKSHORE]     = {10, 20, lTYPE_ALLIANCE},
    [FLIGHTMAP_BARRENS]       = {10, 25, lTYPE_HORDE},
    [FLIGHTMAP_STONETALON]    = {15, 27, lTYPE_CONTESTED},
    [FLIGHTMAP_ASHENVALE]     = {18, 30, lTYPE_CONTESTED},
    [FLIGHTMAP_1KNEEDLES]     = {25, 35, lTYPE_CONTESTED},
    [FLIGHTMAP_DESOLACE]      = {30, 40, lTYPE_CONTESTED},
    [FLIGHTMAP_DUSTWALLOW]    = {35, 45, lTYPE_CONTESTED},
    [FLIGHTMAP_FERALAS]       = {40, 50, lTYPE_CONTESTED},
    [FLIGHTMAP_TANARIS]       = {40, 50, lTYPE_CONTESTED},
    [FLIGHTMAP_AZSHARA]       = {45, 55, lTYPE_CONTESTED},
    [FLIGHTMAP_FELWOOD]       = {48, 55, lTYPE_CONTESTED},
    [FLIGHTMAP_UNGOROCRATER]  = {48, 55, lTYPE_CONTESTED},
    [FLIGHTMAP_SILITHUS]      = {55, 60, lTYPE_CONTESTED},
    [FLIGHTMAP_WINTERSPRING]  = {55, 60, lTYPE_CONTESTED},
    [FLIGHTMAP_TELDRASSIL]    = { 1, 10, lTYPE_ALLIANCE},
    [FLIGHTMAP_MOONGLADE]     = { 1, 60, lTYPE_CONTESTED},
    [FLIGHTMAP_DEADWINDPASS]  = {55, 60, lTYPE_CONTESTED},
};

-- Colours for zones
FLIGHTMAP_COLORS = {
    Unknown   = { r = 0.8, g = 0.8, b = 0.8 },
    Hostile   = { r = 0.9, g = 0.2, b = 0.2 },
    Friendly  = { r = 0.2, g = 0.9, b = 0.2 },
    Contested = { r = 0.8, g = 0.6, b = 0.4 },
};

------------------ Data access functions ------------------

-- Break up a "Town, Zone" string into town and zone components.  If
-- there's no comma, ie, Moonglade, the string will be returned for
-- both town and zone.
local function lFM_GetNameAndZone(taxipath)
    local comma = string.find(taxipath, FLIGHTMAP_SEP_STRING, 1, true);
    if (comma) then
        local name = string.sub(taxipath, 1, comma - 1);
        local zone = string.sub(taxipath,
                comma + string.len(FLIGHTMAP_SEP_STRING),
                -1 - string.len(FLIGHTMAP_SEP_POSTAMBLE));
        return name, zone;
    else
        return taxipath, taxipath;
    end
end

local function lSetDefaultData()
    -- Create an empty knowledge record
    if not FlightMap["Knowledge"] then
        FlightMap.Knowledge = {};
    end

    -- Default option settings
    if (not FlightMap["Opts"]) then
        FlightMap["Opts"] = FLIGHTMAP_DEFAULT_OPTS;
    end

    -- Any options that don't have a value at all should be defaulted
    for k, v in pairs(FLIGHTMAP_DEFAULT_OPTS) do
        if FlightMap.Opts[k] == nil then
            FlightMap.Opts[k] = v;
        end
    end
end

-- Return the flight map data appropriate to the current character
local function lGetFlightMap()
    local _, faction = UnitFactionGroup("player");
    if not faction then return {}; end
    if not FlightMap[faction] then FlightMap[faction] = {}; end
    return FlightMap[faction];
end

-- Returns true if the current character has seen a given node;
-- if the second argument is a true value, teaches the character of
-- the node's existence.
local function lCharacterKnowsNode(node, learning)
    local name = UnitName("player");
    local realm = GetCVar("realmName");

    -- Conglomerate key
    local key = name .. "@" .. realm;

    -- If nothing is known for this character, make it known!
    if not FlightMap.Knowledge[key] then
        FlightMap.Knowledge[key] = {};
    end

    if learning then
        FlightMap.Knowledge[key][node] = true;
    end

    return FlightMap.Knowledge[key][node];
end

-- Return the name of the current map zone
local lZoneNameMap = {};
local function lGetZoneName()
    local function LoadZones(map, ...)
        for i=1, arg.n, 1 do
            map[i] = arg[i];
        end
    end

    local cNum = GetCurrentMapContinent();
    local zNum = GetCurrentMapZone();
    if not lZoneNameMap[cNum] then
        lZoneNameMap[cNum] = {};
        LoadZones(lZoneNameMap[cNum], GetMapZones(cNum));
    end

    return lZoneNameMap[cNum][zNum];
end

local function lRealZoneName(taxiZone, thisCont)
    for k, v in pairs(lZoneNameMap[thisCont]) do
        if string.find(v, taxiZone, 1, true) then
            return v;
        end
    end
    return nil;
end

-- Copy a single node's information from old data structures.
-- This function will recurse through all linked nodes with taxi map
-- coordinates, potentially relearning an entire map from a single node.
local function lMigrateNode(taxiName, taxiX, taxiY, thisCont)
    -- Do nothing if there's no old data
    if not FlightMap.Locs or not FlightMap.Times then
        return false;
    end

    local map = lGetFlightMap();
    local _, faction = UnitFactionGroup("player");
    local name, zone = lFM_GetNameAndZone(taxiName);

    -- Try to work out the real zone name
    local realZone = lRealZoneName(zone, thisCont);
    if not realZone then
        return false;
    end

    -- Need to diddle the zone name for capitals
    for k, v in FLIGHTMAP_SUBZONES do
        if v == realZone then
            realZone = k;
        end
    end

    local nodeName = string.format("%d:%d:%d",
            thisCont, taxiX * 1000, taxiY * 1000);
    local oldName = faction .. ":" .. taxiName;

    -- Migrate base data if it's not already known
    if not map[nodeName] and FlightMap.Locs[oldName] then
        map[nodeName] = {
            Name = taxiName,
            Zone = realZone,
            Continent = thisCont,
            Location = {
                Continent = {
                    x = FlightMap.Locs[oldName].x,
                    y = FlightMap.Locs[oldName].y
                },
                Taxi = {x = taxiX, y = taxiY},
            },
            Flights = {},
            Costs = {},
        };
        -- Take zone coords too, if known
        if FlightMap.Locs[oldName].zx then
            map[nodeName].Location.Zone = {
                x = FlightMap.Locs[oldName].zx,
                y = FlightMap.Locs[oldName].zy
            };
        end
    end

    -- Migrate flight time data if node is now known and there's old time data
    if map[nodeName] and FlightMap.Times[oldName] then
        for dst, dur in pairs(FlightMap.Times[oldName]) do
            -- Is enough information available?
            if FlightMap.Locs[dst] and FlightMap.Locs[dst].mx then
                local destName = string.format("%d:%d:%d", thisCont,
                        FlightMap.Locs[dst].mx * 1000,
                        FlightMap.Locs[dst].my * 1000);
                -- If the node is unknown, migrate it
                if not map[destName] then
                    local destTaxiName = string.sub(dst,
                            string.find(dst, ':') + 1);
                    lMigrateNode(destTaxiName,
                            FlightMap.Locs[dst].mx, FlightMap.Locs[dst].my,
                            thisCont);
                end
                -- Ensure the node is now known before copying time data;
                -- Do not overwrite existing data
                if map[destName] and not map[nodeName].Flights[destName] then
                    map[nodeName].Flights[destName] = dur;
                end
            end
        end
    end

    -- Success!
    return true;
end

-- Migrate old data for currently open taxi map's nodes
-- prerequisite: world map must be set to the correct continent
--               and to any valid zone; taxi map must be open
local function lMigrateNodes()
    local thisCont = GetCurrentMapContinent();
    local nNodes = NumTaxiNodes();
    -- Ensure the zone name map is available for this continent
    local _ = lGetZoneName();
    for i = 1, nNodes, 1 do
        local nodeName = TaxiNodeName(i);
        local tx, ty = TaxiNodePosition(i);
        lMigrateNode(nodeName, tx, ty, thisCont);
    end
end

-- Learn about the currently open taxi map
local function lLearnTaxiNode()
    -- Get the faction appropriate map
    local map = lGetFlightMap();

    -- Save the old map settings
    local oldCont, oldZone = GetCurrentMapContinent(),
                             GetCurrentMapZone();

    -- Ensure the map is set to the right place
    SetMapToCurrentZone();

    -- Migrate old data first
    lMigrateNodes();

    -- Get the current continent number
    local thisCont = GetCurrentMapContinent();

    -- Extract the taxi map information
    local thisNode;
    local destinations = {};
    local numNodes = NumTaxiNodes();
    for index = 1, numNodes, 1 do
        local tType = TaxiNodeGetType(index);
        if (tType == "CURRENT") then
            thisNode = index;
        elseif (tType == "REACHABLE") then
            local mx, my = TaxiNodePosition(index);
            local destName = string.format("%d:%d:%d",
                    thisCont, mx * 1000, my * 1000);

            -- Add to list of destinations
            destinations[destName] = TaxiNodeCost(index);

            -- Note that the character knows of this node
            lCharacterKnowsNode(destName, true);

            -- Update the name; doing this as often as possible
            -- ensures translations are done as soon as a node is
            -- recognised as known
            if map[destName] then
                map[destName].Name = TaxiNodeName(index);
            end
        end
    end

    -- If the current node was found (should always be, but... eh.)
    if (thisNode) then
        -- Establish the coded name of this node
        local mx, my = TaxiNodePosition(thisNode);
        local thisName = string.format("%d:%d:%d",
                thisCont, mx * 1000, my * 1000);
        local zoneName = lGetZoneName();
        local zx, zy = GetPlayerMapPosition("player");
        SetMapZoom(thisCont, nil);
        local cx, cy = GetPlayerMapPosition("player");

        -- Player knows this node now
        lCharacterKnowsNode(thisName, true);

        -- Get, or create, the info structure for the node
        if not map[thisName] then
            map[thisName] = {
                Flights = {},
                Costs = {},
            };
        end

        -- Update all relevant details, to ensure mistakes are fixed
        map[thisName].Name = TaxiNodeName(thisNode);
        map[thisName].Zone = zoneName;
        map[thisName].Continent = thisCont;
        map[thisName].Location = {
            Zone = { x = zx, y = zy },
            Continent = { x = cx, y = cy },
            Taxi = { x = mx, y = my },
        };

        -- Record everywhere this node flies to
        for k,v in pairs(destinations) do
            if not map[thisName].Flights[k] then
                map[thisName].Flights[k] = 0;   -- no duration yet
            end
            -- Create Costs index field if missing (thorarin@tiwaz.org)
            if not map[thisName].Costs then
                map[thisName].Costs = {};
            end
            map[thisName].Costs[k] = v;
        end
    end

    -- Don't mess with the user's choice of zoom!
    SetMapZoom(oldCont, oldZone);
end

-- Returns a table of nodes in a given zone; if second optional
-- argument is present and true, also returns subzone nodes.
local function lGetNodesInZone(zone, subZones)
    local map = lGetFlightMap();
    local nodes = {};
    for k, v in map do
        if v.Zone == zone then
            nodes[k] = v;
        elseif subZones and FLIGHTMAP_SUBZONES[v.Zone] == zone then
            nodes[k] = v;
        end
    end

    return nodes;
end

------------------ Map drawing functions ------------------

local function lFormatMoney(amount)
    local copper = math.mod(amount, 100);
    local silver = math.mod(math.floor(amount/100), 100);
    local gold   = math.floor(amount/10000);

    local result = "";

    -- Only show gold and silver if they have an amount
    if gold > 0 then result = result .. gold .. "g "; end
    if silver > 0 then result = result .. silver .. "s "; end

    -- If gold and silver are both zero, or there's some copper...
    if (gold == 0 and silver == 0) or copper > 0 then
        result = result .. copper .. "c";
    end

    return result;
end

local function lFormatExtra(cost, secs)
    local result = "";
    local separator = "";
    if cost ~= nil and FlightMap.Opts.showCosts then
        local dosh = lFormatMoney(cost);
        if cost == 0 then dosh = FLIGHTMAP_NO_COST; end
        result = result .. dosh;
        separator = " ";
    end
    if secs ~= nil and FlightMap.Opts.showTimes then
        local durn = string.format("%d:%02d", secs / 60, math.mod(secs, 60));
        if secs == 0 then durn = "-:--"; end
        result = result .. separator .. durn;
    end
    return result;
end

-- Add node name, location and all outbound flights with times/costs
-- to FlightMapTooltip, for the given node.  If prefix is non-nil, it
-- will be prepended to all text.  The function will return the total
-- number of flights displayed.
local function lAddFlightsForNode(tooltip, node, prefix)
    -- Sanitize prefix
    if not prefix then prefix = ""; end

    -- Need a map of flight nodes
    local map = lGetFlightMap();

    -- Get the node's data
    local data = map[node];
    if not data then return 0; end
    if not data.Costs then data.Costs = {}; end

    -- Get name of source node
    local name, _ = lFM_GetNameAndZone(data.Name);

    -- And its zone location, if that's known
    local locn = "";
    if data.Location.Zone then
        locn = string.format("%d, %d", data.Location.Zone.x * 100,
                data.Location.Zone.y * 100);
    end

    -- Add that info to the tooltip
    tooltip:AddDoubleLine(prefix .. name, locn);

    -- Increase indentation
    prefix = prefix .. "  ";

    local c = 1;
    -- If it's not known to the current character, say so
    if not lCharacterKnowsNode(node) then
        tooltip:AddLine(prefix .. FLIGHTMAP_NOT_KNOWN, 0.9, 0.4, 0.4);
        c = 0.7;
    end

    local count = 0;
    for dest, secs in data.Flights do
        local destData = map[dest]
        if destData then
            local name, _ = lFM_GetNameAndZone(destData.Name);
            local cost = data.Costs[dest];
            local extra = lFormatExtra(cost, secs);
            if lCharacterKnowsNode(dest) then
                tooltip:AddDoubleLine(prefix .. name, extra,
                        c, c, c, c, c, c);
                count = count + 1;
            elseif FlightMap.Opts.showAllInfo then
                tooltip:AddDoubleLine(prefix .. name, extra,
                        0.7, 0.7, 0.7, 0.7, 0.7, 0.7);
                count = count + 1;
            end
        end
    end

    -- If there were no flights, let it be known
    if count == 0 then
        tooltip:AddLine(prefix .. FLIGHTMAP_NOFLIGHTS, 1, 1, 1);
    end

    return count;
end

-- Update the flight tooltip for a zone
local function lUpdateTooltip(zoneName)
    -- No zone name, no tooltip!
    if not zoneName or zoneName == "" then
        FlightMapTooltip:Hide();
        return;
    end

    -- XXX Is this necessary to repeat each time?  Neither owner nor
    -- anchor should ever change.  But, if it ain't broke...
    FlightMapTooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT");

    -- Determine colour and level range
    local title = FLIGHTMAP_COLORS.Unknown;
    local levels = nil;
    if (FLIGHTMAP_RANGES[zoneName]) then
        local _, faction = UnitFactionGroup("player");
        local min = FLIGHTMAP_RANGES[zoneName][1];
        local max = FLIGHTMAP_RANGES[zoneName][2];
        local side = FLIGHTMAP_RANGES[zoneName][3];
        if (side == lTYPE_CONTESTED) then
            title = FLIGHTMAP_COLORS.Contested;
        else
            if (faction == side) then
                title = FLIGHTMAP_COLORS.Friendly;
            else
                title = FLIGHTMAP_COLORS.Hostile;
            end
        end
        levels = string.format(FLIGHTMAP_LEVELS, min, max);
    end

    -- Show the zone title, add level range if known
    FlightMapTooltip:SetText(zoneName, title.r, title.g, title.b);
    if levels then
        FlightMapTooltip:AddLine(levels, title.r, title.g, title.b);
    end

    -- Discover and add all the flights, including subzones
    local nodes = lGetNodesInZone(zoneName, true);

    -- Outbound flights
    local flights = 0;
    -- FlightMapTooltip:AddLine("\n");
    for node, data in nodes do
        if lCharacterKnowsNode(node) or FlightMap.Opts.showAllInfo then
            flights = flights + lAddFlightsForNode(FlightMapTooltip, node, "");
        end
    end

    -- This stuff seems to get reset each time, possibly by the SetOwner()
    FlightMapTooltip:SetBackdropColor(0, 0, 0, 0.5);
    FlightMapTooltip:SetBackdropBorderColor(0, 0, 0, 0);
    FlightMapTooltip:ClearAllPoints();
    FlightMapTooltip:SetPoint("BOTTOMLEFT", "WorldMapDetailFrame",
            "BOTTOMLEFT", 0, 0);

    -- Only show if there's flight information or level information
    if flights > 0 or levels then
        FlightMapTooltip:Show();
    else
        FlightMapTooltip:Hide();
    end
end

-- Returns true iff an existing world map POI icon is very close to the
-- given coordinates.
local function lCloseToExistingPOI(x, y)
    for i = 1, NUM_WORLDMAP_POIS, 1 do
        local button = getglobal("WorldMapFramePOI" .. i);
        if button:IsVisible() then
            local _, _, index, _, _ = GetMapLandmarkInfo(i);
            -- Index 15 is an invisible POI
            if index ~= 15 then
                local px, py = button:GetCenter();
                px = px - WorldMapDetailFrame:GetLeft();
                py = py - WorldMapDetailFrame:GetBottom();
                if abs(px - x) < FLIGHTMAP_CLOSE and
                abs(py - y) < FLIGHTMAP_CLOSE then
                    return true;
                end
            end
        end
    end
    return false;
end

-- Try showing a POI node; returns true if the POI icon was displayed, or
-- false if it was too close to an existing POI icon, or there were no
-- known coordinates for the requested coordinate space, or the POI number
-- is out of range.
local function lShowNodePOI(node, data, space, num)
    -- Ensure the coordinate space is known
    if not data.Location[space] then return false; end

    -- Ensure the POI number is in range
    if num > FLIGHTMAP_MAX_POIS then return false; end

    -- Get the coordinates
    local x = data.Location[space].x;
    local y = data.Location[space].y;

    -- Convert them to world map pixel-space
    x = x * WorldMapDetailFrame:GetWidth();
    y = (1 - y) * WorldMapDetailFrame:GetHeight();

    -- Ensure the point isn't close to an existing POI icon
    if lCloseToExistingPOI(x, y) then return false; end

    -- Get the node name
    local name, _ = lFM_GetNameAndZone(data.Name);

    -- Get the button
    local button = getglobal("FlightMapPOI" .. num);
    local icon = getglobal("FlightMapPOI" .. num .. "Icon");

    -- Does the user know this flight node?
    if not lCharacterKnowsNode(node) then
        if not FlightMap.Opts.showAllInfo then
            return false;
        end
        icon:SetTexture(FLIGHTMAP_POI_OTHER);
    else
        icon:SetTexture(FLIGHTMAP_POI_KNOWN);
    end

    -- Set all data
    button.name = name;
    button.data = data;
    button.node = node;
    button:SetPoint("CENTER", "WorldMapDetailFrame",
            "BOTTOMLEFT", x, y);
    button:Show();

    -- Done!
    return true;
end

-- Show locations of flight masters for either continent or zone level maps
local function lUpdateFlightPOIs(zoneName)
    local continent = GetCurrentMapContinent();
    local mapZone = GetCurrentMapZone();
    local POI = 1;

    if mapZone ~= 0 and FlightMap.Opts.showPOIs then
        -- Zone level map
        local nodes = lGetNodesInZone(zoneName, false);
        for node, data in nodes do
            if lShowNodePOI(node, data, "Zone", POI) then
                POI = POI + 1;
            end
        end
    elseif continent ~= 0 and FlightMap.Opts.showPOIs then
        -- Continent level map
        local map = lGetFlightMap();
        for node, data in map do
            -- Filter list by continent
            if data.Continent == continent then
                if lShowNodePOI(node, data, "Continent", POI) then
                    POI = POI + 1;
                end
            end
        end
    end

    -- Hide any remaining unused POI buttons
    for i = POI, FLIGHTMAP_MAX_POIS, 1 do
        getglobal("FlightMapPOI" .. i):Hide();
    end
end

-- Draw a line from one flight node to another; returns true if the line
-- was drawn, false if it could not be: number out of range, coordinates
-- not known, etc.
local function lDrawFlightLine(from, to, num)
    -- Check range before doing any real work
    if num > FLIGHTMAP_MAX_PATHS then return false; end

    -- Get the flight map
    local map = lGetFlightMap();

    -- Make sure both ends are known about
    if not map[from] or not map[to] then return false; end

    -- Get the continent coordinate sets
    local src = map[from].Location.Continent;
    local dst = map[to].Location.Continent;

    -- Make sure both are known
    if not src or not dst then return false; end;

    -- Get the texture to work with
    local tex = getglobal("FlightMapPath" .. num);

    -- Reset its anchoring
    tex:ClearAllPoints();

    -- Get the line deltas in pixels
    local dx = abs((src.x - dst.x) * WorldMapDetailFrame:GetWidth());
    local dy = abs((src.y - dst.y) * WorldMapDetailFrame:GetHeight());

    -- Six possible cases:
    -- 1) Horizontal line (dy = 0)
    -- 2) Vertical line (dx = 0)
    -- 3) source is ABOVE and RIGHT of dest
    -- 4) source is BELOW and RIGHT of dest
    -- 5) source is ABOVE and LEFT of dest
    -- 6) source is BELOW and LEFT of dest

    -- Also, the points could be equal; I suspect WoW might crash in this
    -- case, since width would be left at zero... so bail out.
    if dx == 0 and dy == 0 then
        return false;
    end

    -- But, the number of cases can be reduced to four by swapping source
    -- and dest if the source point is LEFT of the destination point:
    if src.x > dst.x then
        local tmp = dst;
        dst = src;
        src = tmp;
    end

    -- And then further reduced to two by making sure no lines really
    -- are horizontal or vertical
    if dy < 1 then dy = 1; end
    if dx < 1 then dx = 1; end

    -- Clip the texture if either delta is smaller than the base size
    -- This prevents the texture from being scaled down, which could result
    -- in parts of the line vanishing during interpolation.
    local clipsize = dx;
    if dy < dx then clipsize = dy; end

    -- Now normalize the clipping size to 0-1
    clipsize = clipsize / FLIGHTMAP_LINE_SIZE;
    if clipsize > 1 then clipsize = 1; end

    local anchorPoint = "NONE";
    if src.y > dst.y then       -- Case 1: source ABOVE dest
        -- Set correct texture
        tex:SetTexture(FLIGHTMAP_TEX_UP);
        -- Use the bottom left corner of it
        tex:SetTexCoord(0, clipsize, 1 - clipsize, 1);
        -- Place the bottom left corner
        anchorPoint = "BOTTOMLEFT";
    else                        -- Case 2: source BELOW dest
        -- Set correct texture
        tex:SetTexture(FLIGHTMAP_TEX_DOWN);
        -- Use the top left corner of it
        tex:SetTexCoord(0, clipsize, 0, clipsize);
        -- Place the top left corner
        anchorPoint = "TOPLEFT";
    end

    -- Set the origin, using whichever anchor point is appropriate
    tex:SetPoint(anchorPoint, "WorldMapDetailFrame", "TOPLEFT",
        src.x * WorldMapDetailFrame:GetWidth(),
        -src.y * WorldMapDetailFrame:GetHeight());

    -- Set the texture's width and height
    tex:SetWidth(dx);
    tex:SetHeight(dy);

    -- Show the texture
    tex:Show();

    -- Success
    return true;
end

-- Fill in flight map lines
local function lDrawFlightLines(zoneName)
    local lineNum = 1;

    -- Only if the zone name is known
    if zoneName and FlightMap.Opts.showPaths then
        -- Iterate over nodes in the current zone
        local nodes = lGetNodesInZone(zoneName, true);
        for node, data in nodes do
            -- If the source node is known
            if FlightMap.Opts.showAllInfo or lCharacterKnowsNode(node) then
                -- ... then iterate over that node's outbound flights
                for dest, duration in data.Flights do
                    -- If the destination node is known
                    if FlightMap.Opts.showAllInfo
                    or lCharacterKnowsNode(dest) then
                        -- ... and the flight line can be drawn
                        if lDrawFlightLine(node, dest, lineNum) then
                            -- ... then increment the line number
                            lineNum = lineNum + 1;
                        end
                    end
                end
            end
        end
    end

    -- Hide remaining flight paths
    for i = lineNum, FLIGHTMAP_MAX_PATHS, 1 do
        getglobal("FlightMapPath" .. i):Hide();
    end
end

-- Last drawn info for tooltip
lFM_CurrentZone = nil;
lFM_CurrentArea = nil;
local lFM_OldUpdate = function() end;

-- Replacement function to draw all the extra goodies of FlightMap
function FlightMap_WorldMapButton_OnUpdate(arg1)
    lFM_OldUpdate(arg1);
    local areaName = WorldMapFrame.areaName;
    local zoneNum = GetCurrentMapZone();

    -- zone name equivalence map
    if FLIGHTMAP_SUBZONES[areaName] then
        areaName = FLIGHTMAP_SUBZONES[areaName];
    end

    -- Bail out if nothing has changed
    if zoneNum == lFM_CurrentZone and areaName == lFM_CurrentArea then
        return;
    end

    -- Continent or zone map?
    if zoneNum == 0 then
        lUpdateTooltip(areaName);
        lUpdateFlightPOIs(areaName);
        lDrawFlightLines(areaName);
    else
        lUpdateFlightPOIs(lGetZoneName());
        lUpdateTooltip(nil);            -- hide it
        lDrawFlightLines(nil);          -- hide them
    end
end

function FlightMapPOIButton_OnEnter()
    local x, y = this:GetCenter();
    local parentX, parentY = WorldMapDetailFrame:GetCenter();
    if (x > parentX) then
        WorldMapTooltip:SetOwner(this, "ANCHOR_LEFT");
    else
        WorldMapTooltip:SetOwner(this, "ANCHOR_RIGHT");
    end
    lAddFlightsForNode(WorldMapTooltip, this.node, "");
    WorldMapTooltip:Show();
end

---------------- Initialization functions -----------------

-- /flightmap handler
function FlightMap_OnSlashCmd()
    if (FlightMapOptionsFrame:IsVisible()) then
        HideUIPanel(FlightMapOptionsFrame);
    else
        ShowUIPanel(FlightMapOptionsFrame);
    end
end

function FlightMap_OnLoad()
    -- Hook TAXIMAP_OPENED to learn flight paths
    this:RegisterEvent("TAXIMAP_OPENED");

    -- Override the world map function
    if (Sea) then
        Sea.util.hook("WorldMapButton_OnUpdate",
                      "FlightMap_WorldMapButton_OnUpdate",
                      "after");
    else
        lFM_OldUpdate = WorldMapButton_OnUpdate;
        WorldMapButton_OnUpdate = FlightMap_WorldMapButton_OnUpdate;
    end

    -- Set up my slash command
    SLASH_FLIGHTMAP1 = "/fmap";
    SLASH_FLIGHTMAP2 = "/flightmap";
    SlashCmdList["FLIGHTMAP"] = FlightMap_OnSlashCmd;

    -- Register for VARIABLES_LOADED to talk to myAddOns
    this:RegisterEvent("VARIABLES_LOADED");

    -- Register the options frame
    UIPanelWindows["FlightMapOptionsFrame"] = {
        area = "center",
        pushable = 0,
    };
end

function FlightMap_OnEvent(event)
    if (event == "TAXIMAP_OPENED") then
        lLearnTaxiNode();
    elseif (event == "VARIABLES_LOADED") then
        lSetDefaultData();

        -- Register with myAddOns
        if (myAddOnsFrame) then
            myAddOnsList.FlightMap = {
                name = FLIGHTMAP_NAME,
                description = FLIGHTMAP_DESCRIPTION,
                version = FLIGHTMAP_VERSION,
                category = MYADDONS_CATEGORY_MAP,
                frame = "FlightMapFrame",
                optionsframe = "FlightMapOptionsFrame",
            };
        end
    end
end

----------------- Options panel functions -----------------

function FlightMapOptionsFrame_OnShow()
    for id, group in pairs(FLIGHTMAP_OPTIONS) do
        local base = "FlightMapOptionsGroup"..id;
        for optid, option in pairs(group.options) do
            local name = base .. "Opt" .. optid;
            local button = getglobal(name);
            local label = getglobal(name .. "Text");
            OptionsFrame_EnableCheckBox(button);

            -- this might not necesarily be true/false!
            if (FlightMap.Opts[option.option]) then
                button:SetChecked(true);
            else
                button:SetChecked(false);
            end

            -- Simple stuff
            label:SetText(option.label);
            button.tooltipText = option.tooltip;
            button.option = option.option;
        end
    end
end

function FlightMapOptionsFrame_OnHide()
    if (MYADDONS_ACTIVE_OPTIONSFRAME == this) then
        ShowUIPanel(myAddOnsFrame);
    end
end

function FlightMapOptionsCheckButton_OnClick()
    if (this:GetChecked()) then
        FlightMap.Opts[this.option] = true;
    else
        FlightMap.Opts[this.option] = false;
    end
end

